/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.impl;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;

import org.ow2.util.archive.api.IArchive;
import org.ow2.util.asm.ClassReader;
import org.ow2.util.scan.api.ArchiveScanner;
import org.ow2.util.scan.api.IArchiveHelper;
import org.ow2.util.scan.api.ScanException;

/**
 * ArchiveScanner.
 * @author Gael Lalire
 */
public final class ArchiveScannerImpl extends ArchiveScanner {

    /**
     * Constructor.
     * @param archive the archive to scan
     */
    public ArchiveScannerImpl(final IArchive archive) {
        super(archive);
    }

    /**
     * Construct an archive scanner with an archive.
     * @param archiveHelper archive helper
     */
    public ArchiveScannerImpl(final IArchiveHelper archiveHelper) {
        super(archiveHelper);
    }

    /**
     * scan the class contained in url.
     * @param url the url
     * @throws ScanException if error occurs
     */
    @Override
    protected void doScan(final URL url) throws ScanException {
        InputStream is = null;
        try {
            URLConnection urlConnection = url.openConnection();
            urlConnection.setDefaultUseCaches(false);
            is = urlConnection.getInputStream();
            new ClassReader(is).accept(new ScanClassVisitor(getArchiveConfigurators()), 0);
        } catch (Exception ioe) {
            throw new ScanException("Error while analyzing file entry '" + url + "' in archive '"
                    + getArchive().getName() + "'", ioe);
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    throw new ScanException("Error while closing input stream of the entry '" + url
                            + "' in archive '" + getArchive().getName() + "'", e);
                }
            }
        }
    }

}
